@@ -3,6 +3,8 @@
 #ifndef __KDTOOLS_H__
 #define __KDTOOLS_H__
 
+#ifndef NO_CXX17 // CRAN
+
 #include <functional>
 #include <algorithm>
 #include <iterator>
@@ -1027,4 +1029,6 @@ void kd_nn_iters(Iter first, Iter last, const TupleType& key, size_t n, OutIter
 } // namespace kdtools
 } // namespace keittlab
 
+#endif // NO_CXX17
+
 #endif // __KDTOOLS_H__
